#import "DerivateView.h"
#import "funciones.h"

extern double sin(double); 
extern double cos(double); 
extern double tan(double);
extern double exp(double); 
extern double log(double);
extern double sinh(double); 
extern double cosh(double); 
extern double tanh(double);
extern double xcuad(double); 

doFunctions doFun[] = {&sin, &cos, &tan, &exp, &log, &sinh, &cosh, &tanh, &xcuad}; 
NSString  *nameFun[] = {@"SIN", @"COS", @"TAN",  @"EXP", @"LOG", @"SINH", @"COSH", @"TANH", @"x^2"}; 
/*
double xcuad(double x)
{
    return x*x; 
}
*/ 

@implementation DerivateView

- (id)initWithFrame:(NSRect)frameRect
{
   NSPoint start; 
   float   t; 
   
   [super initWithFrame:frameRect];
   [self allocateGState];
   start.x = [self bounds].size.width/2.0; 
   start.y = [self bounds].size.height/2.0;
   [self translateOriginToPoint: start]; 

   // Make axes 
   bpaxes = [[NSBezierPath bezierPath] retain]; 
   [bpaxes moveToPoint: NSMakePoint(0.0,  start.y)];
   [bpaxes lineToPoint: NSMakePoint(0.0, -start.y)]; 
   [bpaxes moveToPoint: NSMakePoint( start.x, 0.0)];
   [bpaxes lineToPoint: NSMakePoint(-start.x, 0.0)]; 

   // Make marks in axes 
   for (t=XMARK; t<start.x; t+=XMARK) {
       [bpaxes moveToPoint: NSMakePoint( t,  3)]; 
       [bpaxes lineToPoint: NSMakePoint( t, -3)]; 
       [bpaxes moveToPoint: NSMakePoint(-t,  3)]; 
       [bpaxes lineToPoint: NSMakePoint(-t, -3)]; 
   }
      
   for (t=YMARK; t<start.y; t+=YMARK) {
       [bpaxes moveToPoint: NSMakePoint( 3,  t)]; 
       [bpaxes lineToPoint: NSMakePoint(-3,  t)]; 
       [bpaxes moveToPoint: NSMakePoint( 3, -t)]; 
       [bpaxes lineToPoint: NSMakePoint(-3, -t)]; 
   }

   clean = gder = lfun = laxes = ard = NO; 
   axes = gfun = YES; 
   tfunction = 0; 
   xmin = -PI; 
   xmax = PI; 
   nsteps = 900; 
   return self;
}

- (void)awakeFromNib
{
   [acolor setColor:[NSColor blackColor]]; 
   [bcolor setColor:[NSColor whiteColor]]; 
   [dcolor setColor:[NSColor  blueColor]]; 
   [fcolor setColor:[NSColor   redColor]]; 
   [lcolor setColor:[NSColor blackColor]]; 
}

- (void)dealloc
{
     [bpaxes release]; 
     [super dealloc];
}


- (void)drawRect:(NSRect)rect
{
   [[bcolor color] set]; 
   NSRectFill([self bounds]); 
   
   if (axes) { 
      [[acolor color] set]; 
      [bpaxes stroke]; 
   }
   if (laxes) {
      [[acolor color] set]; 
      [self displayLabelWithRect: rect];
   }

   if (lfun) {
      [[lcolor color] set]; 
      [self displayLabelFunction]; 
   }

   if (!clean) {
      if (gfun==YES && bpfun!=nil) {
         [[fcolor color] set]; 
         [bpfun stroke]; 
      }
      if (gder==YES && bpder!=nil) {
         [[dcolor color] set ];
         [bpder stroke]; 
      }
   }
}

- (void)displayLabelWithRect:(NSRect)rect 
{
   int i=0; 
   double   e, te, lx, ly;
   float fi; 
   NSMutableAttributedString *strYpos;

   lx = rect.size.width/2.0 - 10.0; 
   ly = rect.size.height/2.0 - 5.0; 
   strYpos = [[NSMutableAttributedString alloc] initWithString: @"0"];
   [strYpos addAttribute: NSFontAttributeName
                  value: [NSFont fontWithName:@"Courier" size: 8.0]
                  range: NSMakeRange(0,[strYpos length])];

   [strYpos drawAtPoint: NSMakePoint(-9,-15.0)]; 
   e = 34.0; 
   fi = .5; 
   while (e<lx) {
      [strYpos replaceCharactersInRange:NSMakeRange(0,[strYpos length]) withString:
          [[NSNumber numberWithFloat:fi] stringValue]]; 
      te = e + ((i==1)?4.0:0.0);
      [strYpos drawAtPoint: NSMakePoint(te,-15.0)]; 
       e+=40;
       fi+=.5;
       i = (i==1)?0:1; 
   }
   e = -48.0; 
   fi = -.5;
   while (e > -lx) {
       [strYpos replaceCharactersInRange:NSMakeRange(0,[strYpos length]) withString:
          [[NSNumber numberWithFloat:fi] stringValue]]; 
       [strYpos drawAtPoint: NSMakePoint(e,-15.0)]; 
       fi-=.5;
       e -=40;
   }
   fi = 1; 
   e = 75.0; 
   while (e < ly) {
      [strYpos replaceCharactersInRange:NSMakeRange(0,[strYpos length]) withString:
          [[NSNumber numberWithFloat:fi] stringValue]]; 
      [strYpos drawAtPoint: NSMakePoint(-9,e)];
      e+=80; 
      fi+=1.0; 
   }
   fi = -1; 
   e  = -95.0;
   while(e > -ly) {
      [strYpos replaceCharactersInRange:NSMakeRange(0,[strYpos length]) withString:
          [[NSNumber numberWithFloat:fi] stringValue]]; 
      [strYpos drawAtPoint: NSMakePoint(-9,e)];
      e-=80; 
      fi-=1.0; 
   }
   [strYpos release];
}

- (void)displayLabelFunction
{
   NSMutableAttributedString *name;
   NSPoint p = NSMakePoint(9.0, 15.0);

   name = [[NSMutableAttributedString alloc] initWithString:nameFun[tfunction]];
   [name addAttribute: NSFontAttributeName
                  value: [NSFont fontWithName:@"Times" size: 16.0]
                  range: NSMakeRange(0,[name length])];
   [name drawAtPoint: p];
   [name release]; 
}

- (IBAction)clean:(id)sender
{
   clean = YES; 
   [self display]; 
}

- (IBAction)draw:(id)sender
{
   int i;
   double x, dx; 
   clean = NO; 
   if (bpfun) [bpfun release]; 
   if (bpder) [bpder release]; 
   bpfun = [[NSBezierPath bezierPath] retain]; 
   bpder = [[NSBezierPath bezierPath] retain]; 

   x = xmin; 
   dx = (xmax-xmin)/nsteps; 
   [bpfun moveToPoint: [self function: x]]; 
   [bpder moveToPoint: [self derivate: x]]; 
   for (i=0; i<nsteps; i++) {
       x+=dx; 
       [bpfun lineToPoint: [self function: x]]; 
       [bpder lineToPoint: [self derivate: x]]; 
   }
   [self display]; 
}

- (IBAction)setFunction:(id)sender
{
   tfunction = [[sender selectedCell] tag]; 

   if (ard==YES) [self draw:self]; 
}

- (IBAction)setGraphic:(id)sender
{
   switch([[sender selectedCell] tag]){
   case FUNCTION: gfun = (gfun)?NO:YES; break; 
   case DERIVATE: gder = (gder)?NO:YES; break; 
   }
   [self display]; 
}

- (IBAction)setLabels:(id)sender
{
   switch([[sender selectedCell] tag]) {
   case AXES:           axes  = (axes)?NO:YES; break; 
   case LABEL_AXES:     laxes = (laxes)?NO:YES; break; 
   case LABEL_FUNCTION: lfun  = (lfun)?NO:YES; break; 
   }
   [self display]; 
}

- (IBAction)setColors:(id)sender
{
   [self display]; 
}

- (IBAction)setAutomaticRedraw:(id)sender
{
   ard=(ard)?NO:YES; 
}

- (NSPoint)function: (float)x
{
    return NSMakePoint(x*4*XMARK, (doFun[tfunction](x))*4*YMARK); 
}

- (NSPoint)derivate: (float)t 
{
   double y, dt;
   dt = (xmax-xmin)/(nsteps*100.0); 
   y = (doFun[tfunction](t+dt) - doFun[tfunction](t))/dt; 
   return NSMakePoint(t*4*XMARK, y*4*YMARK);
}

@end
